@@ -14,7 +14,7 @@ from .bug_report_generator import BugReportGenerator
 from .resultSyncer import ResultSyncer
 from .logWatcher import LogWatcher
 from .utils import TimeStamp, getProjectRoot, getLogger
-from .u2Driver import StaticU2UiObject, selector_to_xpath
+from .u2Driver import StaticU2UiObject
 from .fastbotManager import FastbotManager
 import uiautomator2 as u2
 import types
@@ -344,8 +344,6 @@ class KeaTestRunner(TextTestRunner):
                     if self.options.profile_period and self.stepsCount % self.options.profile_period == 0:
                         resultSyncer.sync_event.set()
 
-                    print(f"{len(propsSatisfiedPrecond)} precond satisfied.", flush=True)
-
                     # Go to the next round if no precond satisfied
                     if len(propsSatisfiedPrecond) == 0:
                         continue
@@ -478,18 +476,25 @@ class KeaTestRunner(TextTestRunner):
                     if not precond(test):
                         valid = False
                         break
+                except u2.UiObjectNotFoundError as e:
+                    valid = False
+                    break
                 except Exception as e:
-                    print(f"[ERROR] Error when checking precond: {getFullPropName(test)}", flush=True)
+                    logger.error(f"Error when checking precond: {getFullPropName(test)}")
                     traceback.print_exc()
                     valid = False
                     break
             # if all the precond passed. make it the candidate prop.
             if valid:
-                logger.debug(f"precond satisfied: {getFullPropName(test)}")
                 if result.getExcuted(test) >= getattr(prop, MAX_TRIES_MARKER, float("inf")):
-                    logger.debug(f"{getFullPropName(test)} has reached its max_tries. Skip.")
+                    print(f"{getFullPropName(test)} has reached its max_tries. Skip.", flush=True)
                     continue
                 validProps[propName] = test
+        
+        print(f"{len(validProps)} precond satisfied.", flush=True)
+        if len(validProps) > 0:
+            print("[INFO] Valid properties:",flush=True)
+            print("\n".join([f'                - {getFullPropName(p)}' for p in validProps.values()]), flush=True)
         return validProps
 
     def _logScript(self, execution_info:Dict):
@@ -619,46 +624,56 @@ class KeaTestRunner(TextTestRunner):
            """
         def _get_xpath_widgets(func):
             blocked_set = set()
-            try:
-                script_driver = self.options.Driver.getScriptDriver()
-                preconds = getattr(func, PRECONDITIONS_MARKER, [])
-                if all(precond(script_driver) for precond in preconds):
+            script_driver = self.options.Driver.getScriptDriver()
+            preconds = getattr(func, PRECONDITIONS_MARKER, [])
+
+            def preconds_pass(preconds):
+                try:
+                    return all(precond(script_driver) for precond in preconds)
+                except u2.UiObjectNotFoundError as e:
+                    return False
+                except Exception as e:
+                    logger.error(f"Error processing precond. Check if precond: {e}")
+                    traceback.print_exc()
+                    return False
+
+            if preconds_pass(preconds):
+                try:
                     _widgets = func(self.options.Driver.getStaticChecker())
                     _widgets = _widgets if isinstance(_widgets, list) else [_widgets]
                     for w in _widgets:
                         if isinstance(w, StaticU2UiObject):
-                            xpath = selector_to_xpath(w.selector, True)
-                            blocked_set.add(xpath)
+                            xpath = w.selector_to_xpath(w.selector)
+                            if xpath != '//error':
+                                blocked_set.add(xpath)
                         elif isinstance(w, u2.xpath.XPathSelector):
                             xpath = w._parent.xpath
                             blocked_set.add(xpath)
                         else:
-                            logger.warning(f"{w} Not supported")
-            except Exception as e:
-                logger.error(f"Error processing blocked widgets: {e}")
-                traceback.print_exc()
+                            logger.error(f"block widget defined in {func.__name__} Not supported.")
+                except Exception as e:
+                    logger.error(f"Error processing blocked widgets in: {func}")
+                    logger.error(e)
+                    traceback.print_exc()
             return blocked_set
 
-        res = {
+        result = {
             "widgets": set(),
             "trees": set()
         }
 
-
         for func in self._blockWidgetFuncs["widgets"]:
             widgets = _get_xpath_widgets(func)
-            res["widgets"].update(widgets)
-
+            result["widgets"].update(widgets)
 
         for func in self._blockWidgetFuncs["trees"]:
             trees = _get_xpath_widgets(func)
-            res["trees"].update(trees)
+            result["trees"].update(trees)
 
+        result["widgets"] = list(result["widgets"] - result["trees"])
+        result["trees"] = list(result["trees"])
 
-        res["widgets"] = list(res["widgets"] - res["trees"])
-        res["trees"] = list(res["trees"])
-
-        return res
+        return result
 
 
     def __del__(self):
